#include "muf.h"
#include "resint.h"
#include "pmom.h"
#include "mesq.h"
#include "settings.h"
#include "ccoeff.h"
#include "resconst.h"
#include "expc.h"
#include "hcoeff.h"
#include "anomalous.h"

using namespace pmom;
using namespace resconst;
using namespace ccoeff;

void muf::calc2d(complex <double> b)
{
  double LQ  = resint::LQ;
  double LQ2 = pow(LQ,2);
  double LQ3 = pow(LQ,3);
  double LQ4 = pow(LQ,4);
  double LQ5 = pow(LQ,5);
  double LQ6 = pow(LQ,6);
  double LQ7 = pow(LQ,7);

  double LR  = resint::LR;
  double LR2 = pow(LR,2);

  double LF  = resint::LF;
  double LF2 = pow(LF,2);
  double LF3 = pow(LF,3);
  double LF4 = pow(LF,4);
  double LQF  = LF - LQ;
  double LQF2 = pow(LQF,2);
  double LQF3 = pow(LQF,3);
  double LQF4 = pow(LQF,4);

  double as = resint::aass;
  double as2 = pow(as,2);
  double as3 = pow(as,3);
  double as4 = pow(as,4);
  
  for (int sign = mesq::positive; sign <= mesq::negative; sign++)
    for (int i1 = 0; i1 < mellinint::mdim; i1++)
      for (int i2 = 0; i2 < mellinint::mdim; i2++)
	{
	  int ii1 = anomalous::index(i1,mesq::positive);
	  int ii2 = anomalous::index(i2,sign);
	  int idx = hcoeff::index(i1,i2,sign);

	  //Include exponentiation in the C coefficients
	  complex <double> eC1qq_1   = ccoeff::C1qq_1[ii1];
	  complex <double> eC2qq_1   = ccoeff::C2qq_1[ii1];
	  complex <double> eC1qg_1   = ccoeff::C1qg_1[ii1]*expc::aexpqg_1[ii1]/expc::aexpqq_1[ii1];
	  complex <double> eC2qg_1   = ccoeff::C2qg_1[ii1]*expc::aexpqg_1[ii1]/expc::aexpqq_1[ii1];
	  complex <double> eC2qqp_1  = ccoeff::C2qqp_1[ii1]*expc::aexpqqp_1[ii1]/expc::aexpqq_1[ii1];
	  complex <double> eC2qqb_1  = ccoeff::C2qqb_1[ii1]*expc::aexpqqb_1[ii1]/expc::aexpqq_1[ii1];
	  complex <double> eC2qqbp_1 = ccoeff::C2qqbp_1[ii1]*expc::aexpqqbp_1[ii1]/expc::aexpqq_1[ii1];
	  complex <double> eC1qq_2   = ccoeff::C1qq_2[ii2];
	  complex <double> eC2qq_2   = ccoeff::C2qq_2[ii2];
	  complex <double> eC1qg_2   = ccoeff::C1qg_2[ii2]*expc::aexpqg_2[ii2]/expc::aexpqq_2[ii2];
	  complex <double> eC2qg_2   = ccoeff::C2qg_2[ii2]*expc::aexpqg_2[ii2]/expc::aexpqq_2[ii2];
	  complex <double> eC2qqp_2  = ccoeff::C2qqp_2[ii2]*expc::aexpqqp_2[ii2]/expc::aexpqq_2[ii2];
	  complex <double> eC2qqb_2  = ccoeff::C2qqb_2[ii2]*expc::aexpqqb_2[ii2]/expc::aexpqq_2[ii2];
	  complex <double> eC2qqbp_2 = ccoeff::C2qqbp_2[ii2]*expc::aexpqqbp_2[ii2]/expc::aexpqq_2[ii2];

	    
	  complex <double> H1st_qqb  = (gamma1qq_1[ii1] + gamma1qq_2[ii2])*LQF ;
	  complex <double> H1st_qg_1 = gamma1qg_1[ii1]*LQF;
	  complex <double> H1st_qg_2 = gamma1qg_2[ii2]*LQF;

	  qqb[idx]   += as*H1st_qqb;
	  qg_1[idx]  += as*H1st_qg_1;
	  qg_2[idx]  += as*H1st_qg_2;
	    
	  if (opts.order_hcoef == 1)
	    continue;
	    
	  complex <double> H2st_qqb    = LQF*(eC1qg_1*gamma1gq_1[ii1] + eC1qq_2*gamma1qq_1[ii1] + eC1qg_2*gamma1gq_2[ii2] + eC1qq_2*gamma1qq_2[ii2] + eC1qq_1*(gamma1qq_1[ii1] + gamma1qq_2[ii2]) + gamma2qq_1[ii1] + gamma2qq_2[ii2] + ((gamma1gq_1[ii1]*gamma1qg_1[ii1] + gamma1gq_2[ii2]*gamma1qg_2[ii2] + (gamma1qq_1[ii1] + gamma1qq_2[ii2])*(beta0 + gamma1qq_1[ii1] + gamma1qq_2[ii2]))*LQF)/ 2. - ((gamma1qq_1[ii1] + gamma1qq_2[ii2])*LQ*(2.*B1q - 2.*beta0 + A1q*LQ))/2.);
	  complex <double> H2st_qg_1   = (LQF*(2.*eC1qg_1*(gamma1qq_2[ii2] + gamma1gg_1[ii1]) + 2.*gamma2qg_1[ii1] + gamma1qg_1[ii1]*(2.*eC1qq_2 + 2.*eC1qq_1 + (beta0 + 2.*gamma1qq_2[ii2] + gamma1gg_1[ii1] + gamma1qq_1[ii1])*LF - (2.*B1q - beta0 + 2.*gamma1qq_2[ii2] + gamma1gg_1[ii1] + gamma1qq_1[ii1])*LQ - A1q*LQ2)))/2.;
	  complex <double> H2st_qg_2   = (LQF*(2.*eC1qg_2*(gamma1qq_1[ii1] + gamma1gg_2[ii2]) + 2.*gamma2qg_2[ii2] + gamma1qg_2[ii2]*(2.*eC1qq_1 + 2.*eC1qq_2 + (beta0 + 2.*gamma1qq_1[ii1] + gamma1gg_2[ii2] + gamma1qq_2[ii2])*LF - (2.*B1q - beta0 + 2.*gamma1qq_1[ii1] + gamma1gg_2[ii2] + gamma1qq_2[ii2])*LQ - A1q*LQ2)))/2.;
	  complex <double> H2st_qq_1   = ((2.*eC1qg_1*gamma1gq_1[ii1] + 2.*gamma2qqb_1[ii1] + gamma1gq_1[ii1]*gamma1qg_1[ii1]*LQF)* LQF)/2.;
	  complex <double> H2st_qq_2   = ((2.*eC1qg_2*gamma1gq_2[ii2] + 2.*gamma2qqb_2[ii2] + gamma1gq_2[ii2]*gamma1qg_2[ii2]*LQF)* LQF)/2.;
	  complex <double> H2st_qqp_1  = ((2.*eC1qg_1*gamma1gq_1[ii1] + 2.*gamma2qqbp_1[ii1] + gamma1gq_1[ii1]*gamma1qg_1[ii1]*LQF)* LQF)/2.;
	  complex <double> H2st_qqp_2  = ((2.*eC1qg_2*gamma1gq_2[ii2] + 2.*gamma2qqbp_2[ii2] + gamma1gq_2[ii2]*gamma1qg_2[ii2]*LQF)* LQF)/2.;
	  complex <double> H2st_qqbp_1 = ((2.*eC1qg_1*gamma1gq_1[ii1] + 2.*gamma2qqp_1[ii1] + gamma1gq_1[ii1]*gamma1qg_1[ii1]*LQF)* LQF)/2.;
	  complex <double> H2st_qqbp_2 = ((2.*eC1qg_2*gamma1gq_2[ii2] + 2.*gamma2qqp_2[ii2] + gamma1gq_2[ii2]*gamma1qg_2[ii2]*LQF)* LQF)/2.;
	  complex <double> H2st_gg     = (eC1qg_2*gamma1qg_1[ii1] + gamma1qg_2[ii2]*(eC1qg_1 + gamma1qg_1[ii1]*LQF))*LQF;

	  //Mixed LR*LQF variations
	  H2st_qqb  += - beta0*H1st_qqb*LR;
	  H2st_qg_1 += - beta0*H1st_qg_1*LR;
	  H2st_qg_2 += - beta0*H1st_qg_2*LR;
	    
	  qqb[idx]    += as2*H2st_qqb;
	  qg_1[idx]   += as2*H2st_qg_1;
	  qg_2[idx]   += as2*H2st_qg_2;
	  qq_1[idx]   += as2*H2st_qq_1;
	  qq_2[idx]   += as2*H2st_qq_2;
	  qqp_1[idx]  += as2*H2st_qqp_1;
	  qqp_2[idx]  += as2*H2st_qqp_2;
	  qqbp_1[idx] += as2*H2st_qqbp_1;
	  qqbp_2[idx] += as2*H2st_qqbp_2;
	  gg[idx]     += as2*H2st_gg;
	    
	  if (opts.order_hcoef == 2)
	    continue;

	  double nf = 5.;

	  complex <double> H3st_qqb    = (LQF*(24.*eC1qg_2*eC1qq_1*gamma1gq_2[ii2] + 24.*eC2qg_2*gamma1gq_2[ii2] + 24.*eC1qq_2*eC1qq_1*gamma1qq_2[ii2] + 24.*eC2qq_2*gamma1qq_2[ii2] + 24.*eC2qq_1*gamma1qq_2[ii2] + 24.*eC1qq_2*eC1qg_1*gamma1gq_1[ii1] + 24.*eC2qg_1*gamma1gq_1[ii1] + 24.*eC1qq_2*eC1qq_1*gamma1qq_1[ii1] + 24.*eC2qq_2*gamma1qq_1[ii1] + 24.*eC2qq_1*gamma1qq_1[ii1] + 24.*eC1qg_2*gamma2gq_2[ii2] + 24.*eC1qq_2*gamma2qq_2[ii2] + 24.*eC1qq_1*gamma2qq_2[ii2] + 24.*eC1qg_1*gamma2gq_1[ii1] + 24.*eC1qq_2*gamma2qq_1[ii1] + 24.*eC1qq_1*gamma2qq_1[ii1] + 24.*gamma3qq_2[ii2] + 24.*gamma3qq_1[ii1] + 12.*beta0*eC1qg_2*gamma1gq_2[ii2]*LF + 12.*eC1qg_2*gamma1gg_2[ii2]*gamma1gq_2[ii2]*LF + 12.*eC1qq_2*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LF + 12.*eC1qq_1*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LF + 12.*beta1*gamma1qq_2[ii2]*LF + 12.*beta0*eC1qq_2*gamma1qq_2[ii2]*LF + 12.*beta0*eC1qq_1*gamma1qq_2[ii2]*LF + 12.*eC1qg_2*gamma1gq_2[ii2]*gamma1qq_2[ii2]*LF + 12.*eC1qq_2*pow(gamma1qq_2[ii2],2)*LF + 12.*eC1qq_1*pow(gamma1qq_2[ii2],2)*LF + 12.*beta0*eC1qg_1*gamma1gq_1[ii1]*LF + 24.*eC1qg_1*gamma1qq_2[ii2]*gamma1gq_1[ii1]*LF + 12.*eC1qg_1*gamma1gg_1[ii1]*gamma1gq_1[ii1]*LF + 12.*eC1qq_2*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LF + 12.*eC1qq_1*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LF + 12.*beta1*gamma1qq_1[ii1]*LF + 12.*beta0*eC1qq_2*gamma1qq_1[ii1]*LF + 12.*beta0*eC1qq_1*gamma1qq_1[ii1]*LF + 24.*eC1qg_2*gamma1gq_2[ii2]*gamma1qq_1[ii1]*LF + 24.*eC1qq_2*gamma1qq_2[ii2]*gamma1qq_1[ii1]*LF + 24.*eC1qq_1*gamma1qq_2[ii2]*gamma1qq_1[ii1]*LF + 12.*eC1qg_1*gamma1gq_1[ii1]*gamma1qq_1[ii1]*LF + 12.*eC1qq_2*pow(gamma1qq_1[ii1],2)*LF + 12.*eC1qq_1*pow(gamma1qq_1[ii1],2)*LF + 12.*gamma1qg_2[ii2]*gamma2gq_2[ii2]*LF + 12.*gamma1gq_2[ii2]*gamma2qg_2[ii2]*LF + 24.*beta0*gamma2qq_2[ii2]*LF + 24.*gamma1qq_2[ii2]*gamma2qq_2[ii2]*LF + 24.*gamma1qq_1[ii1]*gamma2qq_2[ii2]*LF + 12.*gamma1qg_1[ii1]*gamma2gq_1[ii1]*LF + 12.*gamma1gq_1[ii1]*gamma2qg_1[ii1]*LF + 24.*beta0*gamma2qq_1[ii1]*LF + 24.*gamma1qq_2[ii2]*gamma2qq_1[ii1]*LF + 24.*gamma1qq_1[ii1]*gamma2qq_1[ii1]*LF + 12.*beta0*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LF2 + 8.*pow(beta0,2)*gamma1qq_2[ii2]*LF2 + 12.*beta0*pow(gamma1qq_2[ii2],2)*LF2 + 12.*beta0*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LF2 + 12.*gamma1qq_2[ii2]*gamma1gq_1[ii1]*gamma1qg_1[ii1]* LF2 + 8.*pow(beta0,2)*gamma1qq_1[ii1]*LF2 + 12.*gamma1gq_2[ii2]*gamma1qg_2[ii2]*gamma1qq_1[ii1]* LF2 + 24.*beta0*gamma1qq_2[ii2]*gamma1qq_1[ii1]*LF2 + 12.*pow(gamma1qq_2[ii2],2)*gamma1qq_1[ii1]* LF2 + 12.*beta0*pow(gamma1qq_1[ii1],2)*LF2 + 12.*gamma1qq_2[ii2]*pow(gamma1qq_1[ii1],2)*LF2 + 4.*(gamma1gg_2[ii2]*gamma1gq_2[ii2]*gamma1qg_2[ii2] + 2.*gamma1gq_2[ii2]*gamma1qg_2[ii2]*gamma1qq_2[ii2] + pow(gamma1qq_2[ii2],3))*LQF2 + 4.*(gamma1gg_1[ii1]*gamma1gq_1[ii1]*gamma1qg_1[ii1] + 2.*gamma1gq_1[ii1]*gamma1qg_1[ii1]*gamma1qq_1[ii1] + pow(gamma1qq_1[ii1],3))*LQF2 - 24.*B1q*eC1qg_2*gamma1gq_2[ii2]*LQ + 36.*beta0*eC1qg_2*gamma1gq_2[ii2]*LQ - 12.*eC1qg_2*gamma1gg_2[ii2]*gamma1gq_2[ii2]*LQ - 12.*eC1qq_2*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LQ - 12.*eC1qq_1*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LQ - 24.*B2q*gamma1qq_2[ii2]*LQ + 12.*beta1*gamma1qq_2[ii2]*LQ - 24.*B1q*eC1qq_2*gamma1qq_2[ii2]*LQ + 36.*beta0*eC1qq_2*gamma1qq_2[ii2]*LQ - 24.*B1q*eC1qq_1*gamma1qq_2[ii2]*LQ + 36.*beta0*eC1qq_1*gamma1qq_2[ii2]*LQ - 12.*eC1qg_2*gamma1gq_2[ii2]*gamma1qq_2[ii2]*LQ - 12.*eC1qq_2*pow(gamma1qq_2[ii2],2)*LQ - 12.*eC1qq_1*pow(gamma1qq_2[ii2],2)*LQ - 24.*B1q*eC1qg_1*gamma1gq_1[ii1]*LQ + 36.*beta0*eC1qg_1*gamma1gq_1[ii1]*LQ - 24.*eC1qg_1*gamma1qq_2[ii2]*gamma1gq_1[ii1]*LQ - 12.*eC1qg_1*gamma1gg_1[ii1]*gamma1gq_1[ii1]*LQ - 12.*eC1qq_2*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LQ - 12.*eC1qq_1*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LQ - 24.*B2q*gamma1qq_1[ii1]*LQ + 12.*beta1*gamma1qq_1[ii1]*LQ - 24.*B1q*eC1qq_2*gamma1qq_1[ii1]*LQ + 36.*beta0*eC1qq_2*gamma1qq_1[ii1]*LQ - 24.*B1q*eC1qq_1*gamma1qq_1[ii1]*LQ + 36.*beta0*eC1qq_1*gamma1qq_1[ii1]*LQ - 24.*eC1qg_2*gamma1gq_2[ii2]*gamma1qq_1[ii1]*LQ - 24.*eC1qq_2*gamma1qq_2[ii2]*gamma1qq_1[ii1]*LQ - 24.*eC1qq_1*gamma1qq_2[ii2]*gamma1qq_1[ii1]*LQ - 12.*eC1qg_1*gamma1gq_1[ii1]*gamma1qq_1[ii1]*LQ - 12.*eC1qq_2*pow(gamma1qq_1[ii1],2)*LQ - 12.*eC1qq_1*pow(gamma1qq_1[ii1],2)*LQ - 12.*gamma1qg_2[ii2]*gamma2gq_2[ii2]*LQ - 12.*gamma1gq_2[ii2]*gamma2qg_2[ii2]*LQ - 24.*B1q*gamma2qq_2[ii2]*LQ + 24.*beta0*gamma2qq_2[ii2]*LQ - 24.*gamma1qq_2[ii2]*gamma2qq_2[ii2]*LQ - 24.*gamma1qq_1[ii1]*gamma2qq_2[ii2]*LQ - 12.*gamma1qg_1[ii1]*gamma2gq_1[ii1]*LQ - 12.*gamma1gq_1[ii1]*gamma2qg_1[ii1]*LQ - 24.*B1q*gamma2qq_1[ii1]*LQ + 24.*beta0*gamma2qq_1[ii1]*LQ - 24.*gamma1qq_2[ii2]*gamma2qq_1[ii1]*LQ - 24.*gamma1qq_1[ii1]*gamma2qq_1[ii1]*LQ - 12.*B1q*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LF*LQ - 12.*B1q*beta0*gamma1qq_2[ii2]*LF*LQ + 8.*pow(beta0,2)*gamma1qq_2[ii2]*LF*LQ - 12.*B1q*pow(gamma1qq_2[ii2],2)*LF*LQ - 12.*B1q*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LF*LQ - 24.*gamma1qq_2[ii2]*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LF* LQ - 12.*B1q*beta0*gamma1qq_1[ii1]*LF*LQ + 8.*pow(beta0,2)*gamma1qq_1[ii1]*LF*LQ - 24.*gamma1gq_2[ii2]*gamma1qg_2[ii2]*gamma1qq_1[ii1]*LF*LQ - 24.*B1q*gamma1qq_2[ii2]*gamma1qq_1[ii1]*LF* LQ - 24.*pow(gamma1qq_2[ii2],2)*gamma1qq_1[ii1]*LF*LQ - 12.*B1q*pow(gamma1qq_1[ii1],2)*LF*LQ - 24.*gamma1qq_2[ii2]*pow(gamma1qq_1[ii1],2)*LF*LQ - 12.*A1q*eC1qg_2*gamma1gq_2[ii2]*LQ2 + 12.*B1q*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LQ2 - 12.*beta0*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LQ2 - 12.*A2q*gamma1qq_2[ii2]*LQ2 + 12.*pow(B1q,2)*gamma1qq_2[ii2]*LQ2 - 24.*B1q*beta0*gamma1qq_2[ii2]*LQ2 + 8.*pow(beta0,2)*gamma1qq_2[ii2]*LQ2 - 12.*A1q*eC1qq_2*gamma1qq_2[ii2]*LQ2 - 12.*A1q*eC1qq_1*gamma1qq_2[ii2]*LQ2 + 12.*B1q*pow(gamma1qq_2[ii2],2)*LQ2 - 12.*beta0*pow(gamma1qq_2[ii2],2)*LQ2 - 12.*A1q*eC1qg_1*gamma1gq_1[ii1]*LQ2 + 12.*B1q*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LQ2 - 12.*beta0*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LQ2 + 12.*gamma1qq_2[ii2]*gamma1gq_1[ii1]*gamma1qg_1[ii1]* LQ2 - 12.*A2q*gamma1qq_1[ii1]*LQ2 + 12.*pow(B1q,2)*gamma1qq_1[ii1]*LQ2 - 24.*B1q*beta0*gamma1qq_1[ii1]*LQ2 + 8.*pow(beta0,2)*gamma1qq_1[ii1]*LQ2 - 12.*A1q*eC1qq_2*gamma1qq_1[ii1]*LQ2 - 12.*A1q*eC1qq_1*gamma1qq_1[ii1]*LQ2 + 12.*gamma1gq_2[ii2]*gamma1qg_2[ii2]*gamma1qq_1[ii1]*LQ2 + 24.*B1q*gamma1qq_2[ii2]*gamma1qq_1[ii1]* LQ2 - 24.*beta0*gamma1qq_2[ii2]*gamma1qq_1[ii1]*LQ2 + 12.*pow(gamma1qq_2[ii2],2)*gamma1qq_1[ii1]* LQ2 + 12.*B1q*pow(gamma1qq_1[ii1],2)*LQ2 - 12.*beta0*pow(gamma1qq_1[ii1],2)*LQ2 + 12.*gamma1qq_2[ii2]*pow(gamma1qq_1[ii1],2)*LQ2 - 12.*A1q*gamma2qq_2[ii2]*LQ2 - 12.*A1q*gamma2qq_1[ii1]*LQ2 - 6.*A1q*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LF*LQ2 - 6.*A1q*beta0*gamma1qq_2[ii2]*LF*LQ2 - 6.*A1q*pow(gamma1qq_2[ii2],2)*LF*LQ2 - 6.*A1q*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LF*LQ2 - 6.*A1q*beta0*gamma1qq_1[ii1]*LF*LQ2 - 12.*A1q*gamma1qq_2[ii2]*gamma1qq_1[ii1]*LF*LQ2 - 6.*A1q*pow(gamma1qq_1[ii1],2)*LF*LQ2 + 6.*A1q*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LQ3 + 12.*A1q*B1q*gamma1qq_2[ii2]*LQ3 - 14.*A1q*beta0*gamma1qq_2[ii2]*LQ3 + 6.*A1q*pow(gamma1qq_2[ii2],2)*LQ3 + 6.*A1q*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LQ3 + 12.*A1q*B1q*gamma1qq_1[ii1]*LQ3 - 14.*A1q*beta0*gamma1qq_1[ii1]*LQ3 + 12.*A1q*gamma1qq_2[ii2]*gamma1qq_1[ii1]*LQ3 + 6.*A1q*pow(gamma1qq_1[ii1],2)*LQ3 + 3.*pow(A1q,2)*gamma1qq_2[ii2]*LQ4 + 3.*pow(A1q,2)*gamma1qq_1[ii1]*LQ4))/24.;
	  complex <double> H3st_qq_1   = (LQF*(12.*eC2qg_1*gamma1gq_1[ii1] + 12.*eC2qqb_1*(gamma1qq_2[ii2] + gamma1qq_1[ii1]) + 12.*eC1qg_1*gamma2gq_1[ii1] + 12.*eC1qq_1*gamma2qqb_1[ii1] + 12.*gamma3qqb_1[ii1] + 6.*beta0*eC1qg_1*gamma1gq_1[ii1]*LF + 12.*eC1qg_1*gamma1qq_2[ii2]*gamma1gq_1[ii1]*LF + 6.*eC1qg_1*gamma1gg_1[ii1]*gamma1gq_1[ii1]*LF + 6.*eC1qq_1*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LF + 6.*eC1qg_1*gamma1gq_1[ii1]*gamma1qq_1[ii1]*LF + 6.*gamma1qg_1[ii1]*gamma2gq_1[ii1]*LF + 6.*gamma1gq_1[ii1]*gamma2qg_1[ii1]*LF + 12.*beta0*gamma2qqb_1[ii1]*LF + 12.*gamma1qq_2[ii2]*gamma2qqb_1[ii1]*LF + 12.*gamma1qq_1[ii1]*gamma2qqb_1[ii1]*LF + 6.*beta0*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LF2 + 6.*gamma1qq_2[ii2]*gamma1gq_1[ii1]*gamma1qg_1[ii1]* LF2 + 2.*gamma1gg_1[ii1]*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LF2 + 4.*gamma1gq_1[ii1]*gamma1qg_1[ii1]*gamma1qq_1[ii1]*LF2 + 6.*eC1qq_2*(2.*eC1qg_1*gamma1gq_1[ii1] + 2.*gamma2qqb_1[ii1] + gamma1gq_1[ii1]*gamma1qg_1[ii1]* LQF) - 12.*B1q*eC1qg_1*gamma1gq_1[ii1]*LQ + 18.*beta0*eC1qg_1*gamma1gq_1[ii1]* LQ - 12.*eC1qg_1*gamma1qq_2[ii2]*gamma1gq_1[ii1]*LQ - 6.*eC1qg_1*gamma1gg_1[ii1]*gamma1gq_1[ii1]* LQ - 6.*eC1qq_1*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LQ - 6.*eC1qg_1*gamma1gq_1[ii1]*gamma1qq_1[ii1]* LQ - 6.*gamma1qg_1[ii1]*gamma2gq_1[ii1]*LQ - 6.*gamma1gq_1[ii1]*gamma2qg_1[ii1]*LQ - 12.*B1q*gamma2qqb_1[ii1]*LQ + 12.*beta0*gamma2qqb_1[ii1]*LQ - 12.*gamma1qq_2[ii2]*gamma2qqb_1[ii1]*LQ - 12.*gamma1qq_1[ii1]*gamma2qqb_1[ii1]*LQ - 6.*B1q*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LF*LQ - 12.*gamma1qq_2[ii2]*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LF* LQ - 4.*gamma1gg_1[ii1]*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LF*LQ - 8.*gamma1gq_1[ii1]*gamma1qg_1[ii1]*gamma1qq_1[ii1]*LF*LQ - 6.*A1q*eC1qg_1*gamma1gq_1[ii1]*LQ2 + 6.*B1q*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LQ2 - 6.*beta0*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LQ2 + 6.*gamma1qq_2[ii2]*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LQ2 + 2.*gamma1gg_1[ii1]*gamma1gq_1[ii1]* gamma1qg_1[ii1]*LQ2 + 4.*gamma1gq_1[ii1]*gamma1qg_1[ii1]*gamma1qq_1[ii1]*LQ2 - 6.*A1q*gamma2qqb_1[ii1]*LQ2 - 3.*A1q*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LF*LQ2 + 3.*A1q*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LQ3))/12.;
	  complex <double> H3st_qqp_1  = (LQF*(12.*eC2qg_1*gamma1gq_1[ii1] + 12.*eC2qqbp_1*(gamma1qq_2[ii2] + gamma1qq_1[ii1]) + 12.*eC1qg_1*gamma2gq_1[ii1] + 12.*eC1qq_1*gamma2qqbp_1[ii1] + 12.*gamma3qqbp_1[ii1] + 6.*beta0*eC1qg_1*gamma1gq_1[ii1]*LF + 12.*eC1qg_1*gamma1qq_2[ii2]*gamma1gq_1[ii1]*LF + 6.*eC1qg_1*gamma1gg_1[ii1]*gamma1gq_1[ii1]*LF + 6.*eC1qq_1*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LF + 6.*eC1qg_1*gamma1gq_1[ii1]*gamma1qq_1[ii1]*LF + 6.*gamma1qg_1[ii1]*gamma2gq_1[ii1]*LF + 6.*gamma1gq_1[ii1]*gamma2qg_1[ii1]*LF + 12.*beta0*gamma2qqbp_1[ii1]*LF + 12.*gamma1qq_2[ii2]*gamma2qqbp_1[ii1]*LF + 12.*gamma1qq_1[ii1]*gamma2qqbp_1[ii1]*LF + 6.*beta0*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LF2 + 6.*gamma1qq_2[ii2]*gamma1gq_1[ii1]*gamma1qg_1[ii1]* LF2 + 2.*gamma1gg_1[ii1]*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LF2 + 4.*gamma1gq_1[ii1]*gamma1qg_1[ii1]*gamma1qq_1[ii1]*LF2 + 6.*eC1qq_2*(2.*eC1qg_1*gamma1gq_1[ii1] + 2.*gamma2qqbp_1[ii1] + gamma1gq_1[ii1]*gamma1qg_1[ii1]*LQF) - 12.*B1q*eC1qg_1*gamma1gq_1[ii1]*LQ + 18.*beta0*eC1qg_1*gamma1gq_1[ii1]*LQ - 12.*eC1qg_1*gamma1qq_2[ii2]*gamma1gq_1[ii1]*LQ - 6.*eC1qg_1*gamma1gg_1[ii1]*gamma1gq_1[ii1]*LQ - 6.*eC1qq_1*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LQ - 6.*eC1qg_1*gamma1gq_1[ii1]*gamma1qq_1[ii1]*LQ - 6.*gamma1qg_1[ii1]*gamma2gq_1[ii1]*LQ - 6.*gamma1gq_1[ii1]*gamma2qg_1[ii1]*LQ - 12.*B1q*gamma2qqbp_1[ii1]*LQ + 12.*beta0*gamma2qqbp_1[ii1]*LQ - 12.*gamma1qq_2[ii2]*gamma2qqbp_1[ii1]*LQ - 12.*gamma1qq_1[ii1]*gamma2qqbp_1[ii1]*LQ - 6.*B1q*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LF*LQ - 12.*gamma1qq_2[ii2]*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LF*LQ - 4.*gamma1gg_1[ii1]*gamma1gq_1[ii1]* gamma1qg_1[ii1]*LF*LQ - 8.*gamma1gq_1[ii1]*gamma1qg_1[ii1]*gamma1qq_1[ii1]*LF*LQ - 6.*A1q*eC1qg_1*gamma1gq_1[ii1]*LQ2 + 6.*B1q*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LQ2 - 6.*beta0*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LQ2 + 6.*gamma1qq_2[ii2]*gamma1gq_1[ii1]*gamma1qg_1[ii1]* LQ2 + 2.*gamma1gg_1[ii1]*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LQ2 + 4.*gamma1gq_1[ii1]*gamma1qg_1[ii1]*gamma1qq_1[ii1]*LQ2 - 6.*A1q*gamma2qqbp_1[ii1]*LQ2 - 3.*A1q*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LF*LQ2 + 3.*A1q*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LQ3))/12.;
	  complex <double> H3st_qqbp_1 = (LQF*(12.*eC2qg_1*gamma1gq_1[ii1] + 12.*eC2qqp_1*(gamma1qq_2[ii2] + gamma1qq_1[ii1]) + 12.*eC1qg_1*gamma2gq_1[ii1] + 12.*eC1qq_1*gamma2qqp_1[ii1] + 12.*gamma3qqp_1[ii1] + 6.*beta0*eC1qg_1*gamma1gq_1[ii1]*LF + 12.*eC1qg_1*gamma1qq_2[ii2]*gamma1gq_1[ii1]*LF + 6.*eC1qg_1*gamma1gg_1[ii1]*gamma1gq_1[ii1]*LF + 6.*eC1qq_1*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LF + 6.*eC1qg_1*gamma1gq_1[ii1]*gamma1qq_1[ii1]*LF + 6.*gamma1qg_1[ii1]*gamma2gq_1[ii1]*LF + 6.*gamma1gq_1[ii1]*gamma2qg_1[ii1]*LF + 12.*beta0*gamma2qqp_1[ii1]*LF + 12.*gamma1qq_2[ii2]*gamma2qqp_1[ii1]*LF + 12.*gamma1qq_1[ii1]*gamma2qqp_1[ii1]*LF + 6.*beta0*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LF2 + 6.*gamma1qq_2[ii2]*gamma1gq_1[ii1]*gamma1qg_1[ii1]* LF2 + 2.*gamma1gg_1[ii1]*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LF2 + 4.*gamma1gq_1[ii1]*gamma1qg_1[ii1]*gamma1qq_1[ii1]*LF2 + 6.*eC1qq_2*(2.*eC1qg_1*gamma1gq_1[ii1] + 2.*gamma2qqp_1[ii1] + gamma1gq_1[ii1]*gamma1qg_1[ii1]* LQF) - 12.*B1q*eC1qg_1*gamma1gq_1[ii1]*LQ + 18.*beta0*eC1qg_1*gamma1gq_1[ii1]* LQ - 12.*eC1qg_1*gamma1qq_2[ii2]*gamma1gq_1[ii1]*LQ - 6.*eC1qg_1*gamma1gg_1[ii1]*gamma1gq_1[ii1]* LQ - 6.*eC1qq_1*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LQ - 6.*eC1qg_1*gamma1gq_1[ii1]*gamma1qq_1[ii1]* LQ - 6.*gamma1qg_1[ii1]*gamma2gq_1[ii1]*LQ - 6.*gamma1gq_1[ii1]*gamma2qg_1[ii1]*LQ - 12.*B1q*gamma2qqp_1[ii1]*LQ + 12.*beta0*gamma2qqp_1[ii1]*LQ - 12.*gamma1qq_2[ii2]*gamma2qqp_1[ii1]*LQ - 12.*gamma1qq_1[ii1]*gamma2qqp_1[ii1]*LQ - 6.*B1q*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LF*LQ - 12.*gamma1qq_2[ii2]*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LF* LQ - 4.*gamma1gg_1[ii1]*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LF*LQ - 8.*gamma1gq_1[ii1]*gamma1qg_1[ii1]*gamma1qq_1[ii1]*LF*LQ - 6.*A1q*eC1qg_1*gamma1gq_1[ii1]*LQ2 + 6.*B1q*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LQ2 - 6.*beta0*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LQ2 + 6.*gamma1qq_2[ii2]*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LQ2 + 2.*gamma1gg_1[ii1]*gamma1gq_1[ii1]* gamma1qg_1[ii1]*LQ2 + 4.*gamma1gq_1[ii1]*gamma1qg_1[ii1]*gamma1qq_1[ii1]*LQ2 - 6.*A1q*gamma2qqp_1[ii1]*LQ2 - 3.*A1q*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LF*LQ2 + 3.*A1q*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LQ3))/12.;
	  complex <double> H3st_qg_1   = (LQF*(24.*eC2qg_1*gamma1qq_2[ii2] + 24.*eC2qg_1*gamma1gg_1[ii1] + 24.*eC2qq_2*gamma1qg_1[ii1] + 24.*eC2qq_1*gamma1qg_1[ii1] + 24.*eC2qqb_1*gamma1qg_1[ii1] - 24.*eC2qqbp_1*gamma1qg_1[ii1] - 24.*eC2qqp_1*gamma1qg_1[ii1] + 24.*eC1qg_1*gamma2qq_2[ii2] + 24.*eC1qg_1*gamma2gg_1[ii1] + 24.*eC1qq_1*gamma2qg_1[ii1] + 24.*gamma3qg_1[ii1] + 12.*eC1qg_1*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LF + 12.*beta0*eC1qg_1*gamma1qq_2[ii2]*LF + 12.*eC1qg_1*pow(gamma1qq_2[ii2],2)*LF + 12.*beta0*eC1qg_1*gamma1gg_1[ii1]*LF + 24.*eC1qg_1*gamma1qq_2[ii2]*gamma1gg_1[ii1]*LF + 12.*eC1qg_1*pow(gamma1gg_1[ii1],2)*LF + 12.*beta1*gamma1qg_1[ii1]*LF + 12.*beta0*eC1qq_1*gamma1qg_1[ii1]*LF + 24.*eC1qq_1*gamma1qq_2[ii2]*gamma1qg_1[ii1]*LF + 12.*eC1qq_1*gamma1gg_1[ii1]*gamma1qg_1[ii1]*LF + 12.*eC1qq_1*gamma1qg_1[ii1]*gamma1qq_1[ii1]*LF + 24.*gamma1qg_1[ii1]*gamma2qq_2[ii2]*LF + 12.*gamma1qg_1[ii1]*gamma2gg_1[ii1]*LF + 24.*beta0*gamma2qg_1[ii1]*LF + 24.*gamma1qq_2[ii2]*gamma2qg_1[ii1]*LF + 12.*gamma1gg_1[ii1]*gamma2qg_1[ii1]*LF + 12.*gamma1qq_1[ii1]*gamma2qg_1[ii1]*LF + 12.*gamma1qg_1[ii1]*gamma2qq_1[ii1]*LF + 12.*gamma1qg_1[ii1]*gamma2qqb_1[ii1]*LF - 12.*gamma1qg_1[ii1]*gamma2qqbp_1[ii1]*LF - 12.*gamma1qg_1[ii1]*gamma2qqp_1[ii1]*LF + 8.*pow(beta0,2)*gamma1qg_1[ii1]*LF2 + 12.*gamma1gq_2[ii2]*gamma1qg_2[ii2]*gamma1qg_1[ii1]*LF2 + 24.*beta0*gamma1qq_2[ii2]*gamma1qg_1[ii1]* LF2 + 12.*pow(gamma1qq_2[ii2],2)*gamma1qg_1[ii1]*LF2 + 12.*beta0*gamma1gg_1[ii1]*gamma1qg_1[ii1]* LF2 + 12.*gamma1qq_2[ii2]*gamma1gg_1[ii1]*gamma1qg_1[ii1]*LF2 + 4.*pow(gamma1gg_1[ii1],2)*gamma1qg_1[ii1]*LF2 + 12.*beta0*gamma1qg_1[ii1]*gamma1qq_1[ii1]*LF2 + 12.*gamma1qq_2[ii2]*gamma1qg_1[ii1]*gamma1qq_1[ii1]*LF2 + 4.*gamma1gg_1[ii1]*gamma1qg_1[ii1]* gamma1qq_1[ii1]*LF2 + 4.*gamma1qg_1[ii1]*pow(gamma1qq_1[ii1],2)*LF2 + 24.*eC1qg_2*gamma1gq_2[ii2]*(eC1qg_1 + gamma1qg_1[ii1]*LQF) - 12.*eC1qg_1*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LQ - 24.*B1q*eC1qg_1*gamma1qq_2[ii2]*LQ + 36.*beta0*eC1qg_1*gamma1qq_2[ii2]*LQ - 12.*eC1qg_1*pow(gamma1qq_2[ii2],2)*LQ - 24.*B1q*eC1qg_1*gamma1gg_1[ii1]*LQ + 36.*beta0*eC1qg_1*gamma1gg_1[ii1]*LQ - 24.*eC1qg_1*gamma1qq_2[ii2]*gamma1gg_1[ii1]*LQ - 12.*eC1qg_1*pow(gamma1gg_1[ii1],2)*LQ - 24.*B2q*gamma1qg_1[ii1]*LQ + 12.*beta1*gamma1qg_1[ii1]*LQ - 24.*B1q*eC1qq_1*gamma1qg_1[ii1]*LQ + 36.*beta0*eC1qq_1*gamma1qg_1[ii1]*LQ - 24.*eC1qq_1*gamma1qq_2[ii2]*gamma1qg_1[ii1]*LQ - 12.*eC1qq_1*gamma1gg_1[ii1]*gamma1qg_1[ii1]*LQ - 12.*eC1qq_1*gamma1qg_1[ii1]*gamma1qq_1[ii1]*LQ - 24.*gamma1qg_1[ii1]*gamma2qq_2[ii2]*LQ - 12.*gamma1qg_1[ii1]*gamma2gg_1[ii1]*LQ - 24.*B1q*gamma2qg_1[ii1]*LQ + 24.*beta0*gamma2qg_1[ii1]*LQ - 24.*gamma1qq_2[ii2]*gamma2qg_1[ii1]*LQ - 12.*gamma1gg_1[ii1]*gamma2qg_1[ii1]*LQ - 12.*gamma1qq_1[ii1]*gamma2qg_1[ii1]*LQ - 12.*gamma1qg_1[ii1]*gamma2qq_1[ii1]*LQ - 12.*gamma1qg_1[ii1]*gamma2qqb_1[ii1]*LQ + 12.*gamma1qg_1[ii1]*gamma2qqbp_1[ii1]*LQ + 12.*gamma1qg_1[ii1]*gamma2qqp_1[ii1]*LQ - 12.*B1q*beta0*gamma1qg_1[ii1]*LF*LQ + 8.*pow(beta0,2)*gamma1qg_1[ii1]*LF*LQ - 24.*gamma1gq_2[ii2]*gamma1qg_2[ii2]*gamma1qg_1[ii1]*LF*LQ - 24.*B1q*gamma1qq_2[ii2]*gamma1qg_1[ii1]*LF* LQ - 24.*pow(gamma1qq_2[ii2],2)*gamma1qg_1[ii1]*LF*LQ - 12.*B1q*gamma1gg_1[ii1]*gamma1qg_1[ii1]*LF* LQ - 24.*gamma1qq_2[ii2]*gamma1gg_1[ii1]*gamma1qg_1[ii1]*LF*LQ - 8.*pow(gamma1gg_1[ii1],2)*gamma1qg_1[ii1]*LF*LQ - 12.*B1q*gamma1qg_1[ii1]*gamma1qq_1[ii1]*LF*LQ - 24.*gamma1qq_2[ii2]*gamma1qg_1[ii1]*gamma1qq_1[ii1]*LF*LQ - 8.*gamma1gg_1[ii1]*gamma1qg_1[ii1]* gamma1qq_1[ii1]*LF*LQ - 8.*gamma1qg_1[ii1]*pow(gamma1qq_1[ii1],2)*LF*LQ - 12.*A1q*eC1qg_1*gamma1qq_2[ii2]*LQ2 - 12.*A1q*eC1qg_1*gamma1gg_1[ii1]*LQ2 - 12.*A2q*gamma1qg_1[ii1]*LQ2 + 12.*pow(B1q,2)*gamma1qg_1[ii1]*LQ2 - 24.*B1q*beta0*gamma1qg_1[ii1]*LQ2 + 8.*pow(beta0,2)*gamma1qg_1[ii1]*LQ2 - 12.*A1q*eC1qq_1*gamma1qg_1[ii1]*LQ2 + 12.*gamma1gq_2[ii2]*gamma1qg_2[ii2]*gamma1qg_1[ii1]*LQ2 + 24.*B1q*gamma1qq_2[ii2]*gamma1qg_1[ii1]*LQ2 - 24.*beta0*gamma1qq_2[ii2]*gamma1qg_1[ii1]*LQ2 + 12.*pow(gamma1qq_2[ii2],2)*gamma1qg_1[ii1]*LQ2 + 12.*B1q*gamma1gg_1[ii1]*gamma1qg_1[ii1]*LQ2 - 12.*beta0*gamma1gg_1[ii1]*gamma1qg_1[ii1]*LQ2 + 12.*gamma1qq_2[ii2]*gamma1gg_1[ii1]*gamma1qg_1[ii1]* LQ2 + 4.*pow(gamma1gg_1[ii1],2)*gamma1qg_1[ii1]*LQ2 + 12.*B1q*gamma1qg_1[ii1]*gamma1qq_1[ii1]* LQ2 - 12.*beta0*gamma1qg_1[ii1]*gamma1qq_1[ii1]*LQ2 + 12.*gamma1qq_2[ii2]*gamma1qg_1[ii1]*gamma1qq_1[ii1]*LQ2 + 4.*gamma1gg_1[ii1]*gamma1qg_1[ii1]* gamma1qq_1[ii1]*LQ2 + 4.*gamma1qg_1[ii1]*pow(gamma1qq_1[ii1],2)*LQ2 - 12.*A1q*gamma2qg_1[ii1]*LQ2 - 6.*A1q*beta0*gamma1qg_1[ii1]*LF*LQ2 - 12.*A1q*gamma1qq_2[ii2]*gamma1qg_1[ii1]*LF*LQ2 - 6.*A1q*gamma1gg_1[ii1]*gamma1qg_1[ii1]*LF*LQ2 - 6.*A1q*gamma1qg_1[ii1]*gamma1qq_1[ii1]*LF*LQ2 + 12.*A1q*B1q*gamma1qg_1[ii1]*LQ3 - 14.*A1q*beta0*gamma1qg_1[ii1]*LQ3 + 12.*A1q*gamma1qq_2[ii2]*gamma1qg_1[ii1]*LQ3 + 6.*A1q*gamma1gg_1[ii1]*gamma1qg_1[ii1]*LQ3 + 6.*A1q*gamma1qg_1[ii1]*gamma1qq_1[ii1]*LQ3 + 3.*pow(A1q,2)*gamma1qg_1[ii1]*LQ4 + 12.*eC1qq_2*(2.*eC1qg_1*(gamma1qq_2[ii2] + gamma1gg_1[ii1]) + 2.*eC1qq_1*gamma1qg_1[ii1] + 2.*gamma2qg_1[ii1] + beta0*gamma1qg_1[ii1]*LF + 2.*gamma1qq_2[ii2]*gamma1qg_1[ii1]*LF + gamma1gg_1[ii1]*gamma1qg_1[ii1]*LF + gamma1qg_1[ii1]*gamma1qq_1[ii1]*LF - 2.*B1q*gamma1qg_1[ii1]*LQ + 3.*beta0*gamma1qg_1[ii1]*LQ - 2.*gamma1qq_2[ii2]*gamma1qg_1[ii1]*LQ - gamma1gg_1[ii1]*gamma1qg_1[ii1]*LQ - gamma1qg_1[ii1]*gamma1qq_1[ii1]*LQ - A1q*gamma1qg_1[ii1]*LQ2) + 24.*eC2qqbp_1*gamma1qg_1[ii1]*nf + 24.*eC2qqp_1*gamma1qg_1[ii1]*nf + 24.*eC1qg_1*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LF*nf + 12.*gamma1qg_1[ii1]*gamma2qqbp_1[ii1]*LF*nf + 12.*gamma1qg_1[ii1]*gamma2qqp_1[ii1]*LF*nf + 8.*gamma1gq_1[ii1]*pow(gamma1qg_1[ii1],2)*LF2*nf - 24.*eC1qg_1*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LQ*nf - 12.*gamma1qg_1[ii1]*gamma2qqbp_1[ii1]*LQ*nf - 12.*gamma1qg_1[ii1]*gamma2qqp_1[ii1]*LQ*nf - 16.*gamma1gq_1[ii1]*pow(gamma1qg_1[ii1],2)*LF*LQ*nf + 8.*gamma1gq_1[ii1]*pow(gamma1qg_1[ii1],2)*LQ2*nf))/24.;
	  complex <double> H3st_qq_2   = (LQF*(12.*eC2qg_2*gamma1gq_2[ii2] + 12.*eC2qqb_2*(gamma1qq_1[ii1] + gamma1qq_2[ii2]) + 12.*eC1qg_2*gamma2gq_2[ii2] + 12.*eC1qq_2*gamma2qqb_2[ii2] + 12.*gamma3qqb_2[ii2] + 6.*beta0*eC1qg_2*gamma1gq_2[ii2]*LF + 12.*eC1qg_2*gamma1qq_1[ii1]*gamma1gq_2[ii2]*LF + 6.*eC1qg_2*gamma1gg_2[ii2]*gamma1gq_2[ii2]*LF + 6.*eC1qq_2*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LF + 6.*eC1qg_2*gamma1gq_2[ii2]*gamma1qq_2[ii2]*LF + 6.*gamma1qg_2[ii2]*gamma2gq_2[ii2]*LF + 6.*gamma1gq_2[ii2]*gamma2qg_2[ii2]*LF + 12.*beta0*gamma2qqb_2[ii2]*LF + 12.*gamma1qq_1[ii1]*gamma2qqb_2[ii2]*LF + 12.*gamma1qq_2[ii2]*gamma2qqb_2[ii2]*LF + 6.*beta0*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LF2 + 6.*gamma1qq_1[ii1]*gamma1gq_2[ii2]*gamma1qg_2[ii2]* LF2 + 2.*gamma1gg_2[ii2]*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LF2 + 4.*gamma1gq_2[ii2]*gamma1qg_2[ii2]*gamma1qq_2[ii2]*LF2 + 6.*eC1qq_1*(2.*eC1qg_2*gamma1gq_2[ii2] + 2.*gamma2qqb_2[ii2] + gamma1gq_2[ii2]*gamma1qg_2[ii2]* LQF) - 12.*B1q*eC1qg_2*gamma1gq_2[ii2]*LQ + 18.*beta0*eC1qg_2*gamma1gq_2[ii2]* LQ - 12.*eC1qg_2*gamma1qq_1[ii1]*gamma1gq_2[ii2]*LQ - 6.*eC1qg_2*gamma1gg_2[ii2]*gamma1gq_2[ii2]* LQ - 6.*eC1qq_2*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LQ - 6.*eC1qg_2*gamma1gq_2[ii2]*gamma1qq_2[ii2]* LQ - 6.*gamma1qg_2[ii2]*gamma2gq_2[ii2]*LQ - 6.*gamma1gq_2[ii2]*gamma2qg_2[ii2]*LQ - 12.*B1q*gamma2qqb_2[ii2]*LQ + 12.*beta0*gamma2qqb_2[ii2]*LQ - 12.*gamma1qq_1[ii1]*gamma2qqb_2[ii2]*LQ - 12.*gamma1qq_2[ii2]*gamma2qqb_2[ii2]*LQ - 6.*B1q*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LF*LQ - 12.*gamma1qq_1[ii1]*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LF* LQ - 4.*gamma1gg_2[ii2]*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LF*LQ - 8.*gamma1gq_2[ii2]*gamma1qg_2[ii2]*gamma1qq_2[ii2]*LF*LQ - 6.*A1q*eC1qg_2*gamma1gq_2[ii2]*LQ2 + 6.*B1q*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LQ2 - 6.*beta0*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LQ2 + 6.*gamma1qq_1[ii1]*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LQ2 + 2.*gamma1gg_2[ii2]*gamma1gq_2[ii2]* gamma1qg_2[ii2]*LQ2 + 4.*gamma1gq_2[ii2]*gamma1qg_2[ii2]*gamma1qq_2[ii2]*LQ2 - 6.*A1q*gamma2qqb_2[ii2]*LQ2 - 3.*A1q*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LF*LQ2 + 3.*A1q*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LQ3))/12.;
	  complex <double> H3st_qqp_2  = (LQF*(12.*eC2qg_2*gamma1gq_2[ii2] + 12.*eC2qqbp_2*(gamma1qq_1[ii1] + gamma1qq_2[ii2]) + 12.*eC1qg_2*gamma2gq_2[ii2] + 12.*eC1qq_2*gamma2qqbp_2[ii2] + 12.*gamma3qqbp_2[ii2] + 6.*beta0*eC1qg_2*gamma1gq_2[ii2]*LF + 12.*eC1qg_2*gamma1qq_1[ii1]*gamma1gq_2[ii2]*LF + 6.*eC1qg_2*gamma1gg_2[ii2]*gamma1gq_2[ii2]*LF + 6.*eC1qq_2*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LF + 6.*eC1qg_2*gamma1gq_2[ii2]*gamma1qq_2[ii2]*LF + 6.*gamma1qg_2[ii2]*gamma2gq_2[ii2]*LF + 6.*gamma1gq_2[ii2]*gamma2qg_2[ii2]*LF + 12.*beta0*gamma2qqbp_2[ii2]*LF + 12.*gamma1qq_1[ii1]*gamma2qqbp_2[ii2]*LF + 12.*gamma1qq_2[ii2]*gamma2qqbp_2[ii2]*LF + 6.*beta0*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LF2 + 6.*gamma1qq_1[ii1]*gamma1gq_2[ii2]*gamma1qg_2[ii2]* LF2 + 2.*gamma1gg_2[ii2]*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LF2 + 4.*gamma1gq_2[ii2]*gamma1qg_2[ii2]*gamma1qq_2[ii2]*LF2 + 6.*eC1qq_1*(2.*eC1qg_2*gamma1gq_2[ii2] + 2.*gamma2qqbp_2[ii2] + gamma1gq_2[ii2]*gamma1qg_2[ii2]*LQF) - 12.*B1q*eC1qg_2*gamma1gq_2[ii2]*LQ + 18.*beta0*eC1qg_2*gamma1gq_2[ii2]*LQ - 12.*eC1qg_2*gamma1qq_1[ii1]*gamma1gq_2[ii2]*LQ - 6.*eC1qg_2*gamma1gg_2[ii2]*gamma1gq_2[ii2]*LQ - 6.*eC1qq_2*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LQ - 6.*eC1qg_2*gamma1gq_2[ii2]*gamma1qq_2[ii2]*LQ - 6.*gamma1qg_2[ii2]*gamma2gq_2[ii2]*LQ - 6.*gamma1gq_2[ii2]*gamma2qg_2[ii2]*LQ - 12.*B1q*gamma2qqbp_2[ii2]*LQ + 12.*beta0*gamma2qqbp_2[ii2]*LQ - 12.*gamma1qq_1[ii1]*gamma2qqbp_2[ii2]*LQ - 12.*gamma1qq_2[ii2]*gamma2qqbp_2[ii2]*LQ - 6.*B1q*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LF*LQ - 12.*gamma1qq_1[ii1]*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LF*LQ - 4.*gamma1gg_2[ii2]*gamma1gq_2[ii2]* gamma1qg_2[ii2]*LF*LQ - 8.*gamma1gq_2[ii2]*gamma1qg_2[ii2]*gamma1qq_2[ii2]*LF*LQ - 6.*A1q*eC1qg_2*gamma1gq_2[ii2]*LQ2 + 6.*B1q*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LQ2 - 6.*beta0*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LQ2 + 6.*gamma1qq_1[ii1]*gamma1gq_2[ii2]*gamma1qg_2[ii2]* LQ2 + 2.*gamma1gg_2[ii2]*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LQ2 + 4.*gamma1gq_2[ii2]*gamma1qg_2[ii2]*gamma1qq_2[ii2]*LQ2 - 6.*A1q*gamma2qqbp_2[ii2]*LQ2 - 3.*A1q*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LF*LQ2 + 3.*A1q*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LQ3))/12.;
	  complex <double> H3st_qqbp_2 = (LQF*(12.*eC2qg_2*gamma1gq_2[ii2] + 12.*eC2qqp_2*(gamma1qq_1[ii1] + gamma1qq_2[ii2]) + 12.*eC1qg_2*gamma2gq_2[ii2] + 12.*eC1qq_2*gamma2qqp_2[ii2] + 12.*gamma3qqp_2[ii2] + 6.*beta0*eC1qg_2*gamma1gq_2[ii2]*LF + 12.*eC1qg_2*gamma1qq_1[ii1]*gamma1gq_2[ii2]*LF + 6.*eC1qg_2*gamma1gg_2[ii2]*gamma1gq_2[ii2]*LF + 6.*eC1qq_2*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LF + 6.*eC1qg_2*gamma1gq_2[ii2]*gamma1qq_2[ii2]*LF + 6.*gamma1qg_2[ii2]*gamma2gq_2[ii2]*LF + 6.*gamma1gq_2[ii2]*gamma2qg_2[ii2]*LF + 12.*beta0*gamma2qqp_2[ii2]*LF + 12.*gamma1qq_1[ii1]*gamma2qqp_2[ii2]*LF + 12.*gamma1qq_2[ii2]*gamma2qqp_2[ii2]*LF + 6.*beta0*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LF2 + 6.*gamma1qq_1[ii1]*gamma1gq_2[ii2]*gamma1qg_2[ii2]* LF2 + 2.*gamma1gg_2[ii2]*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LF2 + 4.*gamma1gq_2[ii2]*gamma1qg_2[ii2]*gamma1qq_2[ii2]*LF2 + 6.*eC1qq_1*(2.*eC1qg_2*gamma1gq_2[ii2] + 2.*gamma2qqp_2[ii2] + gamma1gq_2[ii2]*gamma1qg_2[ii2]* LQF) - 12.*B1q*eC1qg_2*gamma1gq_2[ii2]*LQ + 18.*beta0*eC1qg_2*gamma1gq_2[ii2]* LQ - 12.*eC1qg_2*gamma1qq_1[ii1]*gamma1gq_2[ii2]*LQ - 6.*eC1qg_2*gamma1gg_2[ii2]*gamma1gq_2[ii2]* LQ - 6.*eC1qq_2*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LQ - 6.*eC1qg_2*gamma1gq_2[ii2]*gamma1qq_2[ii2]* LQ - 6.*gamma1qg_2[ii2]*gamma2gq_2[ii2]*LQ - 6.*gamma1gq_2[ii2]*gamma2qg_2[ii2]*LQ - 12.*B1q*gamma2qqp_2[ii2]*LQ + 12.*beta0*gamma2qqp_2[ii2]*LQ - 12.*gamma1qq_1[ii1]*gamma2qqp_2[ii2]*LQ - 12.*gamma1qq_2[ii2]*gamma2qqp_2[ii2]*LQ - 6.*B1q*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LF*LQ - 12.*gamma1qq_1[ii1]*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LF* LQ - 4.*gamma1gg_2[ii2]*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LF*LQ - 8.*gamma1gq_2[ii2]*gamma1qg_2[ii2]*gamma1qq_2[ii2]*LF*LQ - 6.*A1q*eC1qg_2*gamma1gq_2[ii2]*LQ2 + 6.*B1q*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LQ2 - 6.*beta0*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LQ2 + 6.*gamma1qq_1[ii1]*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LQ2 + 2.*gamma1gg_2[ii2]*gamma1gq_2[ii2]* gamma1qg_2[ii2]*LQ2 + 4.*gamma1gq_2[ii2]*gamma1qg_2[ii2]*gamma1qq_2[ii2]*LQ2 - 6.*A1q*gamma2qqp_2[ii2]*LQ2 - 3.*A1q*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LF*LQ2 + 3.*A1q*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LQ3))/12.;
	  complex <double> H3st_qg_2   = (LQF*(24.*eC2qg_2*gamma1qq_1[ii1] + 24.*eC2qg_2*gamma1gg_2[ii2] + 24.*eC2qq_1*gamma1qg_2[ii2] + 24.*eC2qq_2*gamma1qg_2[ii2] + 24.*eC2qqb_2*gamma1qg_2[ii2] - 24.*eC2qqbp_2*gamma1qg_2[ii2] - 24.*eC2qqp_2*gamma1qg_2[ii2] + 24.*eC1qg_2*gamma2qq_1[ii1] + 24.*eC1qg_2*gamma2gg_2[ii2] + 24.*eC1qq_2*gamma2qg_2[ii2] + 24.*gamma3qg_2[ii2] + 12.*eC1qg_2*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LF + 12.*beta0*eC1qg_2*gamma1qq_1[ii1]*LF + 12.*eC1qg_2*pow(gamma1qq_1[ii1],2)*LF + 12.*beta0*eC1qg_2*gamma1gg_2[ii2]*LF + 24.*eC1qg_2*gamma1qq_1[ii1]*gamma1gg_2[ii2]*LF + 12.*eC1qg_2*pow(gamma1gg_2[ii2],2)*LF + 12.*beta1*gamma1qg_2[ii2]*LF + 12.*beta0*eC1qq_2*gamma1qg_2[ii2]*LF + 24.*eC1qq_2*gamma1qq_1[ii1]*gamma1qg_2[ii2]*LF + 12.*eC1qq_2*gamma1gg_2[ii2]*gamma1qg_2[ii2]*LF + 12.*eC1qq_2*gamma1qg_2[ii2]*gamma1qq_2[ii2]*LF + 24.*gamma1qg_2[ii2]*gamma2qq_1[ii1]*LF + 12.*gamma1qg_2[ii2]*gamma2gg_2[ii2]*LF + 24.*beta0*gamma2qg_2[ii2]*LF + 24.*gamma1qq_1[ii1]*gamma2qg_2[ii2]*LF + 12.*gamma1gg_2[ii2]*gamma2qg_2[ii2]*LF + 12.*gamma1qq_2[ii2]*gamma2qg_2[ii2]*LF + 12.*gamma1qg_2[ii2]*gamma2qq_2[ii2]*LF + 12.*gamma1qg_2[ii2]*gamma2qqb_2[ii2]*LF - 12.*gamma1qg_2[ii2]*gamma2qqbp_2[ii2]*LF - 12.*gamma1qg_2[ii2]*gamma2qqp_2[ii2]*LF + 8.*pow(beta0,2)*gamma1qg_2[ii2]*LF2 + 12.*gamma1gq_1[ii1]*gamma1qg_1[ii1]*gamma1qg_2[ii2]*LF2 + 24.*beta0*gamma1qq_1[ii1]*gamma1qg_2[ii2]* LF2 + 12.*pow(gamma1qq_1[ii1],2)*gamma1qg_2[ii2]*LF2 + 12.*beta0*gamma1gg_2[ii2]*gamma1qg_2[ii2]* LF2 + 12.*gamma1qq_1[ii1]*gamma1gg_2[ii2]*gamma1qg_2[ii2]*LF2 + 4.*pow(gamma1gg_2[ii2],2)*gamma1qg_2[ii2]*LF2 + 12.*beta0*gamma1qg_2[ii2]*gamma1qq_2[ii2]*LF2 + 12.*gamma1qq_1[ii1]*gamma1qg_2[ii2]*gamma1qq_2[ii2]*LF2 + 4.*gamma1gg_2[ii2]*gamma1qg_2[ii2]* gamma1qq_2[ii2]*LF2 + 4.*gamma1qg_2[ii2]*pow(gamma1qq_2[ii2],2)*LF2 + 24.*eC1qg_1*gamma1gq_1[ii1]*(eC1qg_2 + gamma1qg_2[ii2]*LQF) - 12.*eC1qg_2*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LQ - 24.*B1q*eC1qg_2*gamma1qq_1[ii1]*LQ + 36.*beta0*eC1qg_2*gamma1qq_1[ii1]*LQ - 12.*eC1qg_2*pow(gamma1qq_1[ii1],2)*LQ - 24.*B1q*eC1qg_2*gamma1gg_2[ii2]*LQ + 36.*beta0*eC1qg_2*gamma1gg_2[ii2]*LQ - 24.*eC1qg_2*gamma1qq_1[ii1]*gamma1gg_2[ii2]*LQ - 12.*eC1qg_2*pow(gamma1gg_2[ii2],2)*LQ - 24.*B2q*gamma1qg_2[ii2]*LQ + 12.*beta1*gamma1qg_2[ii2]*LQ - 24.*B1q*eC1qq_2*gamma1qg_2[ii2]*LQ + 36.*beta0*eC1qq_2*gamma1qg_2[ii2]*LQ - 24.*eC1qq_2*gamma1qq_1[ii1]*gamma1qg_2[ii2]*LQ - 12.*eC1qq_2*gamma1gg_2[ii2]*gamma1qg_2[ii2]*LQ - 12.*eC1qq_2*gamma1qg_2[ii2]*gamma1qq_2[ii2]*LQ - 24.*gamma1qg_2[ii2]*gamma2qq_1[ii1]*LQ - 12.*gamma1qg_2[ii2]*gamma2gg_2[ii2]*LQ - 24.*B1q*gamma2qg_2[ii2]*LQ + 24.*beta0*gamma2qg_2[ii2]*LQ - 24.*gamma1qq_1[ii1]*gamma2qg_2[ii2]*LQ - 12.*gamma1gg_2[ii2]*gamma2qg_2[ii2]*LQ - 12.*gamma1qq_2[ii2]*gamma2qg_2[ii2]*LQ - 12.*gamma1qg_2[ii2]*gamma2qq_2[ii2]*LQ - 12.*gamma1qg_2[ii2]*gamma2qqb_2[ii2]*LQ + 12.*gamma1qg_2[ii2]*gamma2qqbp_2[ii2]*LQ + 12.*gamma1qg_2[ii2]*gamma2qqp_2[ii2]*LQ - 12.*B1q*beta0*gamma1qg_2[ii2]*LF*LQ + 8.*pow(beta0,2)*gamma1qg_2[ii2]*LF*LQ - 24.*gamma1gq_1[ii1]*gamma1qg_1[ii1]*gamma1qg_2[ii2]*LF*LQ - 24.*B1q*gamma1qq_1[ii1]*gamma1qg_2[ii2]*LF* LQ - 24.*pow(gamma1qq_1[ii1],2)*gamma1qg_2[ii2]*LF*LQ - 12.*B1q*gamma1gg_2[ii2]*gamma1qg_2[ii2]*LF* LQ - 24.*gamma1qq_1[ii1]*gamma1gg_2[ii2]*gamma1qg_2[ii2]*LF*LQ - 8.*pow(gamma1gg_2[ii2],2)*gamma1qg_2[ii2]*LF*LQ - 12.*B1q*gamma1qg_2[ii2]*gamma1qq_2[ii2]*LF*LQ - 24.*gamma1qq_1[ii1]*gamma1qg_2[ii2]*gamma1qq_2[ii2]*LF*LQ - 8.*gamma1gg_2[ii2]*gamma1qg_2[ii2]* gamma1qq_2[ii2]*LF*LQ - 8.*gamma1qg_2[ii2]*pow(gamma1qq_2[ii2],2)*LF*LQ - 12.*A1q*eC1qg_2*gamma1qq_1[ii1]*LQ2 - 12.*A1q*eC1qg_2*gamma1gg_2[ii2]*LQ2 - 12.*A2q*gamma1qg_2[ii2]*LQ2 + 12.*pow(B1q,2)*gamma1qg_2[ii2]*LQ2 - 24.*B1q*beta0*gamma1qg_2[ii2]*LQ2 + 8.*pow(beta0,2)*gamma1qg_2[ii2]*LQ2 - 12.*A1q*eC1qq_2*gamma1qg_2[ii2]*LQ2 + 12.*gamma1gq_1[ii1]*gamma1qg_1[ii1]*gamma1qg_2[ii2]*LQ2 + 24.*B1q*gamma1qq_1[ii1]*gamma1qg_2[ii2]*LQ2 - 24.*beta0*gamma1qq_1[ii1]*gamma1qg_2[ii2]*LQ2 + 12.*pow(gamma1qq_1[ii1],2)*gamma1qg_2[ii2]*LQ2 + 12.*B1q*gamma1gg_2[ii2]*gamma1qg_2[ii2]*LQ2 - 12.*beta0*gamma1gg_2[ii2]*gamma1qg_2[ii2]*LQ2 + 12.*gamma1qq_1[ii1]*gamma1gg_2[ii2]*gamma1qg_2[ii2]* LQ2 + 4.*pow(gamma1gg_2[ii2],2)*gamma1qg_2[ii2]*LQ2 + 12.*B1q*gamma1qg_2[ii2]*gamma1qq_2[ii2]* LQ2 - 12.*beta0*gamma1qg_2[ii2]*gamma1qq_2[ii2]*LQ2 + 12.*gamma1qq_1[ii1]*gamma1qg_2[ii2]*gamma1qq_2[ii2]*LQ2 + 4.*gamma1gg_2[ii2]*gamma1qg_2[ii2]* gamma1qq_2[ii2]*LQ2 + 4.*gamma1qg_2[ii2]*pow(gamma1qq_2[ii2],2)*LQ2 - 12.*A1q*gamma2qg_2[ii2]*LQ2 - 6.*A1q*beta0*gamma1qg_2[ii2]*LF*LQ2 - 12.*A1q*gamma1qq_1[ii1]*gamma1qg_2[ii2]*LF*LQ2 - 6.*A1q*gamma1gg_2[ii2]*gamma1qg_2[ii2]*LF*LQ2 - 6.*A1q*gamma1qg_2[ii2]*gamma1qq_2[ii2]*LF*LQ2 + 12.*A1q*B1q*gamma1qg_2[ii2]*LQ3 - 14.*A1q*beta0*gamma1qg_2[ii2]*LQ3 + 12.*A1q*gamma1qq_1[ii1]*gamma1qg_2[ii2]*LQ3 + 6.*A1q*gamma1gg_2[ii2]*gamma1qg_2[ii2]*LQ3 + 6.*A1q*gamma1qg_2[ii2]*gamma1qq_2[ii2]*LQ3 + 3.*pow(A1q,2)*gamma1qg_2[ii2]*LQ4 + 12.*eC1qq_1*(2.*eC1qg_2*(gamma1qq_1[ii1] + gamma1gg_2[ii2]) + 2.*eC1qq_2*gamma1qg_2[ii2] + 2.*gamma2qg_2[ii2] + beta0*gamma1qg_2[ii2]*LF + 2.*gamma1qq_1[ii1]*gamma1qg_2[ii2]*LF + gamma1gg_2[ii2]*gamma1qg_2[ii2]*LF + gamma1qg_2[ii2]*gamma1qq_2[ii2]*LF - 2.*B1q*gamma1qg_2[ii2]*LQ + 3.*beta0*gamma1qg_2[ii2]*LQ - 2.*gamma1qq_1[ii1]*gamma1qg_2[ii2]*LQ - gamma1gg_2[ii2]*gamma1qg_2[ii2]*LQ - gamma1qg_2[ii2]*gamma1qq_2[ii2]*LQ - A1q*gamma1qg_2[ii2]*LQ2) + 24.*eC2qqbp_2*gamma1qg_2[ii2]*nf + 24.*eC2qqp_2*gamma1qg_2[ii2]*nf + 24.*eC1qg_2*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LF*nf + 12.*gamma1qg_2[ii2]*gamma2qqbp_2[ii2]*LF*nf + 12.*gamma1qg_2[ii2]*gamma2qqp_2[ii2]*LF*nf + 8.*gamma1gq_2[ii2]*pow(gamma1qg_2[ii2],2)*LF2*nf - 24.*eC1qg_2*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LQ*nf - 12.*gamma1qg_2[ii2]*gamma2qqbp_2[ii2]*LQ*nf - 12.*gamma1qg_2[ii2]*gamma2qqp_2[ii2]*LQ*nf - 16.*gamma1gq_2[ii2]*pow(gamma1qg_2[ii2],2)*LF*LQ*nf + 8.*gamma1gq_2[ii2]*pow(gamma1qg_2[ii2],2)*LQ2*nf))/24.;

	  complex <double> H3st_gg     = (LQF*(2.*eC2qg_2*gamma1qg_1[ii1] + 2.*eC2qg_1*gamma1qg_2[ii2] + 2.*eC1qg_2*gamma2qg_1[ii1] + beta0*eC1qg_2*gamma1qg_1[ii1]*LF + eC1qg_2*gamma1gg_1[ii1]*gamma1qg_1[ii1]*LF + eC1qg_2*gamma1qg_1[ii1]*gamma1qq_1[ii1]*LF + 2.*eC1qg_2*gamma1qg_1[ii1]*gamma1gg_2[ii2]*LF + 2.*eC1qq_2*gamma1qg_1[ii1]*gamma1qg_2[ii2]*LF + 2.*gamma1qg_2[ii2]*gamma2qg_1[ii1]*LF + 2.*gamma1qg_1[ii1]*gamma2qg_2[ii2]*LF + 2.*beta0*gamma1qg_1[ii1]*gamma1qg_2[ii2]*LF2 + gamma1gg_1[ii1]*gamma1qg_1[ii1]*gamma1qg_2[ii2]*LF2 + gamma1qg_1[ii1]*gamma1qq_1[ii1]*gamma1qg_2[ii2]* LF2 + gamma1qg_1[ii1]*gamma1gg_2[ii2]*gamma1qg_2[ii2]*LF2 + gamma1qg_1[ii1]*gamma1qg_2[ii2]*gamma1qq_2[ii2]*LF2 - 2.*B1q*eC1qg_2*gamma1qg_1[ii1]*LQ + 3.*beta0*eC1qg_2*gamma1qg_1[ii1]*LQ - eC1qg_2*gamma1gg_1[ii1]*gamma1qg_1[ii1]*LQ - eC1qg_2*gamma1qg_1[ii1]*gamma1qq_1[ii1]*LQ - 2.*eC1qg_2*gamma1qg_1[ii1]*gamma1gg_2[ii2]*LQ - 2.*eC1qq_2*gamma1qg_1[ii1]*gamma1qg_2[ii2]*LQ - 2.*gamma1qg_2[ii2]*gamma2qg_1[ii1]*LQ - 2.*gamma1qg_1[ii1]*gamma2qg_2[ii2]*LQ - 2.*B1q*gamma1qg_1[ii1]*gamma1qg_2[ii2]*LF*LQ - 2.*gamma1gg_1[ii1]*gamma1qg_1[ii1]*gamma1qg_2[ii2]*LF*LQ - 2.*gamma1qg_1[ii1]*gamma1qq_1[ii1]* gamma1qg_2[ii2]*LF*LQ - 2.*gamma1qg_1[ii1]*gamma1gg_2[ii2]*gamma1qg_2[ii2]*LF*LQ - 2.*gamma1qg_1[ii1]*gamma1qg_2[ii2]*gamma1qq_2[ii2]*LF*LQ - A1q*eC1qg_2*gamma1qg_1[ii1]*LQ2 + 2.*B1q*gamma1qg_1[ii1]*gamma1qg_2[ii2]*LQ2 - 2.*beta0*gamma1qg_1[ii1]*gamma1qg_2[ii2]*LQ2 + gamma1gg_1[ii1]*gamma1qg_1[ii1]*gamma1qg_2[ii2]*LQ2 + gamma1qg_1[ii1]*gamma1qq_1[ii1]*gamma1qg_2[ii2]* LQ2 + gamma1qg_1[ii1]*gamma1gg_2[ii2]*gamma1qg_2[ii2]*LQ2 + gamma1qg_1[ii1]*gamma1qg_2[ii2]*gamma1qq_2[ii2]*LQ2 - A1q*gamma1qg_1[ii1]*gamma1qg_2[ii2]*LF*LQ2 + A1q*gamma1qg_1[ii1]*gamma1qg_2[ii2]*LQ3 + 2.*eC1qq_1*gamma1qg_1[ii1]* (eC1qg_2 + gamma1qg_2[ii2]*LF - gamma1qg_2[ii2]*LQ) + eC1qg_1*(2.*eC1qg_2*(gamma1gg_1[ii1] + gamma1gg_2[ii2]) + 2.*eC1qq_2*gamma1qg_2[ii2] + 2.*gamma2qg_2[ii2] + beta0*gamma1qg_2[ii2]*LF + 2.*gamma1gg_1[ii1]*gamma1qg_2[ii2]*LF + gamma1gg_2[ii2]*gamma1qg_2[ii2]*LF + gamma1qg_2[ii2]*gamma1qq_2[ii2]*LF - 2.*B1q*gamma1qg_2[ii2]*LQ + 3.*beta0*gamma1qg_2[ii2]*LQ - 2.*gamma1gg_1[ii1]*gamma1qg_2[ii2]* LQ - gamma1gg_2[ii2]*gamma1qg_2[ii2]*LQ - gamma1qg_2[ii2]*gamma1qq_2[ii2]*LQ - A1q*gamma1qg_2[ii2]*LQ2)))/2.;
	  complex <double> H3st_qbg_1  = ((2.*eC2qqb_2*gamma1qg_1[ii1] + (2.*eC1qg_2*gamma1gq_2[ii2] + 2.*gamma2qqb_2[ii2] + gamma1gq_2[ii2]*gamma1qg_2[ii2]*LQF)*(eC1qg_1 + gamma1qg_1[ii1]*LQF))* LQF)/2.;
	  complex <double> H3st_qbg_2  = ((2.*eC2qqb_1*gamma1qg_2[ii2] + (2.*eC1qg_1*gamma1gq_1[ii1] + 2.*gamma2qqb_1[ii1] + gamma1gq_1[ii1]*gamma1qg_1[ii1]*LQF)*(eC1qg_2 + gamma1qg_2[ii2]*LQF))* LQF)/2.;
	  complex <double> H3st_qpg_1  = ((2.*eC2qqp_2*gamma1qg_1[ii1] + (2.*eC1qg_2*gamma1gq_2[ii2] + 2.*gamma2qqp_2[ii2] + gamma1gq_2[ii2]*gamma1qg_2[ii2]*LQF)*(eC1qg_1 + gamma1qg_1[ii1]*LQF))* LQF)/2.;
	  complex <double> H3st_qpg_2  = ((2.*eC2qqp_1*gamma1qg_2[ii2] + (2.*eC1qg_1*gamma1gq_1[ii1] + 2.*gamma2qqp_1[ii1] + gamma1gq_1[ii1]*gamma1qg_1[ii1]*LQF)*(eC1qg_2 + gamma1qg_2[ii2]*LQF))* LQF)/2.;
	  complex <double> H3st_qbpg_1 = ((2.*eC2qqbp_2*gamma1qg_1[ii1] + (2.*eC1qg_2*gamma1gq_2[ii2] + 2.*gamma2qqbp_2[ii2] + gamma1gq_2[ii2]*gamma1qg_2[ii2]*LQF)*(eC1qg_1 + gamma1qg_1[ii1]*LQF))* LQF)/2.;
	  complex <double> H3st_qbpg_2 = ((2.*eC2qqbp_1*gamma1qg_2[ii2] + (2.*eC1qg_1*gamma1gq_1[ii1] + 2.*gamma2qqbp_1[ii1] + gamma1gq_1[ii1]*gamma1qg_1[ii1]*LQF)*(eC1qg_2 + gamma1qg_2[ii2]*LQF))* LQF)/2.;

	  //Mixed LR*LQF variations
	  H3st_qqb    += -2*beta0*H2st_qqb*LR - (beta1*LR + pow(beta0,2)*LR2)*H1st_qqb;
	  H3st_qq_1   += -2*beta0*H2st_qq_1*LR;
	  H3st_qq_2   += -2*beta0*H2st_qq_2*LR;
	  H3st_qqp_1  += -2*beta0*H2st_qqp_1*LR;
	  H3st_qqp_2  += -2*beta0*H2st_qqp_2*LR;
	  H3st_qqbp_1 += -2*beta0*H2st_qqbp_1*LR;
	  H3st_qqbp_2 += -2*beta0*H2st_qqbp_2*LR;
	  H3st_qg_1   += -2*beta0*H2st_qg_1*LR - (beta1*LR + pow(beta0,2)*LR2)*H1st_qg_1;
	  H3st_qg_2   += -2*beta0*H2st_qg_2*LR - (beta1*LR + pow(beta0,2)*LR2)*H1st_qg_2;
	  H3st_gg     += -2*beta0*H2st_gg*LR;
	    
	  qqb[idx]    += as3*H3st_qqb;
	  qg_1[idx]   += as3*H3st_qg_1;
	  qg_2[idx]   += as3*H3st_qg_2;
	  qq_1[idx]   += as3*H3st_qq_1;
	  qq_2[idx]   += as3*H3st_qq_2;
	  qqp_1[idx]  += as3*H3st_qqp_1;
	  qqp_2[idx]  += as3*H3st_qqp_2;
	  qqbp_1[idx] += as3*H3st_qqbp_1;
	  qqbp_2[idx] += as3*H3st_qqbp_2;
	  gg[idx]     += as3*H3st_gg;
	  qbg_1[idx]  += as3*H3st_qbg_1;
	  qbg_2[idx]  += as3*H3st_qbg_2;
	  qpg_1[idx]  += as3*H3st_qpg_1;
	  qpg_2[idx]  += as3*H3st_qpg_2;
	  qbpg_1[idx] += as3*H3st_qbpg_1;
	  qbpg_2[idx] += as3*H3st_qbpg_2;
	}

  for (int sign = mesq::positive; sign <= mesq::negative; sign++)
    for (int i = 0; i < mellinint::mdim; i++)
      {
	int idx = pmom::index(i,sign);
	qqb[idx]  *= expc::qqb[idx];
	qg_1[idx] *= expc::qqb[idx];
	qg_2[idx] *= expc::qqb[idx];
	qq_1[idx] *= expc::qqb[idx];
	qq_2[idx] *= expc::qqb[idx];
	qqp_1[idx]*= expc::qqb[idx];
	qqp_2[idx]*= expc::qqb[idx];
	qqbp_1[idx] *= expc::qqb[idx];
	qqbp_2[idx] *= expc::qqb[idx];
	gg[idx]   *= expc::qqb[idx];
	qbg_1[idx]  *= expc::qqb[idx];
	qbg_2[idx]  *= expc::qqb[idx];
	qpg_1[idx]  *= expc::qqb[idx];
	qpg_2[idx]  *= expc::qqb[idx];
	qbpg_1[idx] *= expc::qqb[idx];
	qbpg_2[idx] *= expc::qqb[idx];
	qbq[idx]  *= expc::qqb[idx];
	qpqbp[idx]*= expc::qqb[idx];
	qbpqp[idx]*= expc::qqb[idx];
	qbqp_1[idx] *= expc::qqb[idx];
	qbqp_2[idx] *= expc::qqb[idx];
	qbqbp_1[idx]*= expc::qqb[idx];
	qbqbp_2[idx]*= expc::qqb[idx];
	qpqp_1[idx] *= expc::qqb[idx];
	qpqp_2[idx] *= expc::qqb[idx];
      }      
    
}
