#include "sparserules.h"
#include "sandia_rules.hpp"
#include "sparse_grid_gl.hpp"
#include "sgmg.hpp"
#include "settings.h"

#include <iostream>
#include <iomanip>

double *sr::x3;
double *sr::w3;
int sr::n3;

double *sr::x4;
double *sr::w4;
int sr::n4;

typedef void ( *GWPointer ) ( int order, int np, double p[], double w[] );

void sr::init()
{
  init3();
  init4();
}

void sr::init3()
{
  /*
  int dim_num = 3;
  int level = 3; //4; //take from input
  //Determine the number of points.
  n = sparse_grid_gl_size ( dim_num, level );

  cout << "\n";
  cout << "  Number of unique points in the grid = " << n << "\n";

  //  Allocate space for the weights and points.
  w = new double[n];
  x = new double[dim_num*n];

  //  Compute the weights and points.
  sparse_grid_gl ( dim_num, level, n, w, x );
  */

  //double tol = -1.;
  double tol = 1e-7;

  int dim_num = 3;
  int level_max = opts.level;
  int np[3] = {0,0,0};
  int np_sum = 0;
  double *p;

  int rule[3];
  int growth[3];
  GWPointer gw_compute_points[3];
  GWPointer gw_compute_weights[3];
  for (int i = 0; i < dim_num; i++)
    {
      rule[i] = 4; //1; //4 is Gauss-Legendre rule, see https://people.math.sc.edu/Burkardt/cpp_src/sgmg/sgmg.html
      growth[i] = 6; //1; //6; //6 is Full exponential growth, see https://people.math.sc.edu/Burkardt/cpp_src/sgmg/sgmg.html
      gw_compute_points[i] = webbur::legendre_compute_points_np; //webbur::clenshaw_curtis_compute_points_np;
      gw_compute_weights[i] = webbur::legendre_compute_weights_np; //webbur::clenshaw_curtis_compute_weights_np;
    }

  //Compute necessary data.
  int point_total_num = webbur::sgmg_size_total(dim_num, level_max, rule, growth);
  n3 = webbur::sgmg_size (dim_num, level_max, rule, np, p, gw_compute_points, tol, growth);

  int *sparse_unique_index = new int[point_total_num];
  webbur::sgmg_unique_index (dim_num, level_max, rule, np, p, gw_compute_points, tol, n3, point_total_num, growth, sparse_unique_index);

  int *sparse_order = new int[dim_num*n3];
  int *sparse_index = new int[dim_num*n3];
  webbur::sgmg_index (dim_num, level_max, rule, n3, point_total_num, sparse_unique_index, growth, sparse_order, sparse_index);

  //Compute points and weights.
  x3 = new double[dim_num * n3];
  webbur::sgmg_point (dim_num, level_max, rule, np, p, gw_compute_points, n3, sparse_order, sparse_index, growth, x3);

  w3 = new double[n3];
  webbur::sgmg_weight (dim_num, level_max, rule, np, p, gw_compute_weights, n3, point_total_num, sparse_unique_index, growth, w3);

  //Write points and weights to files.
  //string file_name = "sgmg_d3_l2_ccxglxgh";
  //webbur::sgmg_write (dim_num, rule, np, p, n, w, x, file_name);

  delete [] sparse_index;
  delete [] sparse_order;
  delete [] sparse_unique_index;

  /*
  //check rule
  cout << endl;
  cout << "  Grid weights:" << endl;
  cout << endl;
  for (int point = 0; point < n; point++ )
  {
    cout << "  " << setw(4)  << point
         << "  " << fixed << setprecision(6) << setw(10) 
	 << w[point] << endl;
  }
  
  cout << endl;
  cout << "  Grid points:\n";
  cout << endl;
  for (int point = 0; point < n; point++)
  {
    
    cout << "  " << setw(4) << point;
    for (int dim = 0; dim < dim_num; dim++)
    {
      cout << "  " << fixed << setprecision(6) << setw(9) 
           << x[dim+point*dim_num];
    }
    cout << endl;
  }
  double tot = 0.;
  for (unsigned i = 0; i < n; i++)
    tot += w[i];
  cout << "sumweights " << tot << endl;
  */
}

void sr::init4()
{
  /*
  int dim_num = 3;
  int level = 3; //4; //take from input
  //Determine the number of points.
  n = sparse_grid_gl_size ( dim_num, level );

  cout << "\n";
  cout << "  Number of unique points in the grid = " << n << "\n";

  //  Allocate space for the weights and points.
  w = new double[n];
  x = new double[dim_num*n];

  //  Compute the weights and points.
  sparse_grid_gl ( dim_num, level, n, w, x );
  */

  double tol = 1e-7;

  int dim_num = 4;
  int level_max = opts.level;
  int np[4] = {0,0,0,0};
  int np_sum = 0;
  double *p;

  int rule[4];
  int growth[4];
  GWPointer gw_compute_points[4];
  GWPointer gw_compute_weights[4];
  for (int i = 0; i < dim_num; i++)
    {
      rule[i] = 4; //1;
      growth[i] = 6; //1; //6;
      gw_compute_points[i] = webbur::legendre_compute_points_np; //webbur::clenshaw_curtis_compute_points_np;
      gw_compute_weights[i] = webbur::legendre_compute_weights_np; //webbur::clenshaw_curtis_compute_weights_np;
    }

  //Compute necessary data.
  int point_total_num = webbur::sgmg_size_total(dim_num, level_max, rule, growth);
  n4 = webbur::sgmg_size (dim_num, level_max, rule, np, p, gw_compute_points, tol, growth);

  int *sparse_unique_index = new int[point_total_num];
  webbur::sgmg_unique_index (dim_num, level_max, rule, np, p, gw_compute_points, tol, n4, point_total_num, growth, sparse_unique_index);

  int *sparse_order = new int[dim_num*n4];
  int *sparse_index = new int[dim_num*n4];
  webbur::sgmg_index (dim_num, level_max, rule, n4, point_total_num, sparse_unique_index, growth, sparse_order, sparse_index);

  //Compute points and weights.
  x4 = new double[dim_num * n4];
  webbur::sgmg_point (dim_num, level_max, rule, np, p, gw_compute_points, n4, sparse_order, sparse_index, growth, x4);

  w4 = new double[n4];
  webbur::sgmg_weight (dim_num, level_max, rule, np, p, gw_compute_weights, n4, point_total_num, sparse_unique_index, growth, w4);

  //Write points and weights to files.
  //string file_name = "sgmg_d3_l2_ccxglxgh";
  //webbur::sgmg_write (dim_num, rule, np, p, n, w, x, file_name);

  delete [] sparse_index;
  delete [] sparse_order;
  delete [] sparse_unique_index;

  /*
  //check rule
  cout << endl;
  cout << "  Grid weights:" << endl;
  cout << endl;
  for (int point = 0; point < n; point++ )
  {
    cout << "  " << setw(4)  << point
         << "  " << fixed << setprecision(6) << setw(10) 
	 << w[point] << endl;
  }
  
  cout << endl;
  cout << "  Grid points:\n";
  cout << endl;
  for (int point = 0; point < n; point++)
  {
    
    cout << "  " << setw(4) << point;
    for (int dim = 0; dim < dim_num; dim++)
    {
      cout << "  " << fixed << setprecision(6) << setw(9) 
           << x[dim+point*dim_num];
    }
    cout << endl;
  }
  double tot = 0.;
  for (unsigned i = 0; i < n; i++)
    tot += w[i];
  cout << "sumweights " << tot << endl;
  */
}

void sr::release()
{
  delete [] x3;
  delete [] w3;

  delete [] x4;
  delete [] w4;
}
